#include "esp_camera.h"
#include <WiFi.h>
#include <ArduinoWebsockets.h>
#include <HTTPClient.h>

#include "SPI.h"
#include "driver/rtc_io.h"
#include "ESP32_MailClient.h"
#include <FS.h>
#include <SPIFFS.h>


//
// WARNING!!! Make sure that you have either selected ESP32 Wrover Module,
//            or another board which has PSRAM enabled
//

// Select camera model
//#define CAMERA_MODEL_WROVER_KIT
//#define CAMERA_MODEL_ESP_EYE
//#define CAMERA_MODEL_M5STACK_PSRAM
//#define CAMERA_MODEL_M5STACK_WIDE
#define CAMERA_MODEL_AI_THINKER

#include "camera_pins.h"

const char* ssid = "goldin_lab1";
const char* password = "goldin53743001";
const char* websocket_server_host = "60.250.156.230";
const uint16_t websocket_server_port = 8093;

String IFTTTUrl="http://maker.ifttt.com/trigger/Test/with/key/c3xo5EvpBX64fPEqxphcR4jBTzDh1r2joTDsB_BslOA";

// Replace with your data
#define emailSenderAccount    "service.gitcr081@gmail.com@gmail.com"
#define emailSenderPassword   "yang53743001"
#define smtpServer            "smtp.gmail.com"
#define smtpServerPort        465
#define emailSubject          "ESP32-CAM Picture"
#define emailRecipient        "service.gitcr081@@gmail.com"

// The Email Sending data object contains config and data to send
SMTPData smtpData;

// Picture file name to save in SPIFFS
#define PICTURE_FILE "/picture.jpg"

void startCameraServer();
using namespace websockets;
WebsocketsClient client;

//Set Static IP Address
IPAddress local_IP(192, 168, 50, 83);
//Set Gateway IP Address
IPAddress gateway(192, 168, 50, 254);
IPAddress subnet(255, 255, 255, 0);
IPAddress dns(8, 8, 8, 8); // Google DNS

const char* getphoto ="getphoto";
const char* statusok ="statusok";

void setup() {  
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0); // We disable brownout detector
  
  Serial.begin(115200);
  Serial.setDebugOutput(true);
  Serial.println();
  
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 10000000;
  config.pixel_format = PIXFORMAT_JPEG;
  //init with high specs to pre-allocate larger buffers
  if(psramFound()){
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }
  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    return;
  }
  capturePictureSaveSpiffs();
  sendPicture();
  
  sensor_t * s = esp_camera_sensor_get();
  // initial sensors are flipped vertically and colors are a bit saturated
  if (s->id.PID == OV3660_PID) {
    s->set_vflip(s, 1); // flip it back
    s->set_brightness(s, 2); // up the brightness just a bit
    s->set_saturation(s, -2); // lower the saturation
    s->set_contrast(s, 2);       // -2 to 2
  }
  // drop down frame size for higher initial frame rate
  s->set_framesize(s, FRAMESIZE_QVGA);

#if defined(CAMERA_MODEL_M5STACK_WIDE) || defined(CAMERA_MODEL_M5STACK_ESP32CAM)  
  s->set_hmirror(s, 1);        // 0 = disable , 1 = enable
  s->set_vflip(s, 0);          // 0 = disable , 1 = enable
  s->set_brightness(s, -2);
  s->set_framesize(s, FRAMESIZE_QVGA);
#endif

  // We initialize  SPI Flash File System (SPIFFS) to save the last picture taken with the ESP32-CAM:
  if (!SPIFFS.begin(true)) {
    Serial.println("An Error has occurred while mounting SPIFFS");
    ESP.restart();
  }
  else {
    delay(500);
    Serial.println("SPIFFS is mounted!");
  }
  
  client.send("Hi Server!");
  if(!WiFi.config(local_IP, gateway, subnet,dns)) {
  Serial.println("Set IP Failed");
}
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  startCameraServer();
  Serial.print("Camera Ready! Use 'http://");
  Serial.print(WiFi.localIP());
  Serial.println("' to connect");

  while(!client.connect(websocket_server_host, websocket_server_port, "/")){
    delay(500);
    Serial.print(".");
  }
  Serial.println("Websocket Connected!");
}
void capturePictureSaveSpiffs( void ) {
  camera_fb_t * fb = NULL; // pointer
  bool ok = 0; // Boolean indicating if the picture has been taken correctly

  do {
    // We take a picture with the ESP32-CAM
    Serial.println("Taking a picture...");

    fb = esp_camera_fb_get();
    if (!fb) {
      Serial.println("Failed to take a picture!");
      return;
    }

    Serial.printf("Picture file name is %s\n", PICTURE_FILE);
    File file = SPIFFS.open(PICTURE_FILE, FILE_WRITE);

    // We insert the data in the picture file
    if (!file) {
      Serial.println("Failed to open file in writing mode");
    }
    else {
      file.write(fb->buf, fb->len); // payload (image), payload length
      Serial.print("The picture has been saved in ");
      Serial.print(PICTURE_FILE);
      Serial.print(" - Size: ");
      Serial.print(file.size());
      Serial.println(" bytes");
    }
    // We close the file
    file.close();
    esp_camera_fb_return(fb);

    // We check if file has been correctly saved in SPIFFS
    ok = checkPicture(SPIFFS);
  } while ( !ok );
}

// We check if picture capture was successful
bool checkPicture( fs::FS &fs ) {
  File f_pic = fs.open( PICTURE_FILE );
  unsigned int pic_sz = f_pic.size();
  return ( pic_sz > 100 );
}
// Send picture function
void sendPicture( void ) {
  Serial.println("Sending of an email is in progress...");
  // We set the SMTP Server Email host, port, account and password
  smtpData.setLogin(smtpServer, smtpServerPort, emailSenderAccount, emailSenderPassword);

  // We set the sender name and email
  smtpData.setSender("ESP32-CAM", emailSenderAccount);

  // We set email priority or importance High, Normal, Low or 1 to 5 (1 is highest)
  smtpData.setPriority("High");

  // We set the subject
  smtpData.setSubject(emailSubject);

  // We set the email message in HTML format
  smtpData.setMessage("<h2>Picture taken with ESP32-CAM attached in this email.</h2>", true);
  // uncomment this line if you want to set the email message in text format
  //smtpData.setMessage("Picture taken with ESP32-CAM attached in this email.", false);

  // We add recipients, you can have more than one recipient
  smtpData.addRecipient(emailRecipient);
  //smtpData.addRecipient(emailRecipient1);

  // We attach files from SPIFFS
  smtpData.addAttachFile(PICTURE_FILE, "image/jpg");
  // We set the storage type to attach files in your email (SPIFFS)
  smtpData.setFileStorageType(MailClientStorageType::SPIFFS);
  // We set a callback function that will be called upon sending an email
  smtpData.setSendCallback(sendCallback);

  // We start sending an email
  if (!MailClient.sendMail(smtpData))
    Serial.println("Error sending Email, " + MailClient.smtpErrorReason());

  // We clear all data from an object to free up a memory
  smtpData.empty();
}

// Callback function to get the email sending status
void sendCallback(SendStatus msg) {
  // We print the current status
  Serial.println(msg.info());
}


void loop() {
  camera_fb_t *fb = esp_camera_fb_get();  
  if(!fb){
    Serial.println("Camera capture failed");
    esp_camera_fb_return(fb);
    return;
  }

  if(fb->format != PIXFORMAT_JPEG){
    Serial.println("Non-JPEG data not implemented");
    return;
  }
  
  client.sendBinary((const char*) fb->buf, fb->len);
  esp_camera_fb_return(fb);
  
  if ((WiFi.status()!= WL_CONNECTED)){
    WiFi.reconnect();
    Serial.println("WiFi reconnected");
  }
  while(!client.connect(websocket_server_host, websocket_server_port, "/")){
    delay(500);
    Serial.print(".");
  }
  Serial.println("Websocket Connected!");



  String url=IFTTTUrl+"?value1="+getphoto+"&value2="+statusok; 
  //client.poll();
  //Start to send data to IFTTT
    HTTPClient http;
    Serial.print("[HTTP] begin...\n");
    http.begin(url); //HTTP

    Serial.print("[HTTP] GET...\n");
    // start connection and send HTTP header
    int httpCode = http.GET();
    // httpCode will be negative on error
    if(httpCode > 0) {
        // HTTP header has been send and Server response header has been handled
        Serial.printf("[HTTP] GET... code: %d\n", httpCode);
        // file found at server
        if(httpCode == HTTP_CODE_OK) {
            String payload = http.getString();
            Serial.println(payload);
        }
    } else {
        Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
    }
    http.end();

  //delay(1000);//send data every 20 seconds

  
     
}
